function [IAFet,CSFet]=iafet(fe,facfrec,ft,C,nolin,estab)

% IAFET calcula la cantidad de informacion (en unidades relativas) 
% asignada a la zona de frecuencia espacial fe, frecuencia temporal
% ft y contraste C por el sistema visual, segun una dependencia:
% 
%                         K                   K
%   IAFet(fe,ft,C)= ------------ = -------------------------
%                    Afe Aft AC      Cu(fe,ft) + f(fe,ft,C)
%
% Donde:
%
%     Afe = Umbrales incrementales de frecuencia espacial
%
%     Aft = Umbrales incrementales de frecuencia temporal
%
%     AC  = Umbrales incrementales de contraste
%
% y consideramos que Afe y Aft son constantes.
% Asi mismo, consideramos que los umbrales incrementales de contraste vienen 
% dados por la suma de dos terminos:
%
%     Cu(fe,ft)  = Termino independiente de C
%                  Umbral absoluto de contraste para la frecuencia fe,ft
%                  Calculado segun la superficie espacio-temporal de Kelly
%                  NOTA: Como en la expresion de Kelly hay un log(ft/fe), 
%                        cuando ft o fe sean cero, se sustituiran por 0.00001
%                        para evitar singularidades. 
%
%     f(fe,ft,C) = Termino DEPENDIENTE de C
%                  Da cuenta de la no-linealidad de la respuesta del sistema a 
%                  contrastes, o lo que es lo mismo, de la no-uniformidad de la
%                  sensibilidad en contrastes. 
%                  Es de sobra conocido que la respuesta (en el caso de ft=0)
%                  es no lineal (existen expresiones explicitas para modelizar
%                  esta no linealidad), pero no existen datos sobre dichas 
%                  no-linealidades para frecuencias temporales no nulas.
%                  (o al menos yo no los conozco... quiza Pascual si).
%
%                  Aqui supondremos (lo cual es mucho suponer) que la no linea-
%                  lidad es analoga a la que se da para ft=0.
%
% Podemos no considerar la no-linealidad y trabajar solo con el termino constante:
% el contraste umbral de Kelly Cu(fe,ft): aplic_no_lineal?
% Podemos 'elegir' una determinada asignacion de bits por coeficiente frecuencial,
% es decir, seleccionar una asignacion igual a la que surge de considerar la no li-
% nealidad, pero imponiendo una distribucion lineal en amplitud. (si se pone un 0
% en aplic_no_lineal?, se puede poner un 1 en ciaf_no_lin?)
% 
% Para una determinada frecuencia temporal, IAFET permite calcular la IAF espacio-
% temporal para M frecuencias espaciales y N contrastes, de forma que, genera una
% matriz M*N con el resultado para cada frecuencia espacial en cada fila.
% 
% NOTAS:
%
%   * 'fe' debe ser un vector columna y C debe ser un vector fila!
%
%   * El factor 'facfrec' actua sobre el dominio de frecuencias espaciales en el
%     momento del calculo de la CSFet (Kelly 79), de forma que permite controlar
%     ligeramente el ancho de banda de la CSF: Si facfrec>1 se reduce la banda 
%     de paso y si facfrec<1 se aumenta.
%     (Para obtener Elect. Lett. facfrec=0.75-0.8)
%     Esta es una forma chapucera pero rapida de ampliar la banda de paso espacial 
%     de la CSFet de Kelly 79 (que no concuerda con Kelly 86 donde aparece mayor).
%
% USO: [IAFet,CSFet]=iafet(fe,facfrec,ft,C,[aplic_no_lineal?(0/1) ciaf_no_lin? aplic_log?],estabiliz?(0/1));
% 

if length(nolin)==1
   nolin=[nolin nolin];
end

nolini=nolin;
nolin=nolini(1);

if ((nolini(1)==0)&(nolini(2)==1))
   nolin=1;      
end
   
   fe=fe+0.00001*(fe==0);
   
   % Para ft menores que 0.4 la CSF espacio temporal de Kelly del 79 no se reduce a la 
   % CSF espacial de Kelly del 85, con lo que utilizamos la expresion sin dependencia de
   % ft cuando las frecuencias temporales son lo suficientemente bajas (TODO ESTO EN CASO
   % DE ESTABILIZACION).
   
   if estab==1
      if ft>0.3   
         fec=facfrec*fe;
         CSFet=(250/1896.4)*(ft.*fec*4*pi^2).*(6.1+7.3*abs(log(ft./(3*fec))).^3).*exp(-(4*pi*(ft+2*fec)/45.9));
         CSFet=0.000001*(CSFet==0)+CSFet;
         c1=(1./CSFet)>1;
         c2=(1./CSFet)<1;
         CSFet=CSFet.*c2+c1;
         Cu=1./CSFet;
         Cu=Cu*ones(1,length(C));
   %      Cu=1./CSFet;
   %      Cu=Cu.*(Cu<1)+(Cu>=1);
   %      Cu=Cu*ones(1,length(C));
      else
         fm=3;           % Frec. max. sensib.
         p=1;            % Anch. banda
         cum=0.004;      % Minimo contraste umbral
         fe=facfrec*fe;
         CSFet=((fe/fm).^p).*exp(-(fe/fm));
         CSFet=CSFet/maxi(CSFet);
         CSFet=(1/cum)*CSFet;
         CSFet=0.000001*(CSFet==0)+CSFet;
         c1=(1./CSFet)>1;
         c2=(1./CSFet)<1;
         CSFet=CSFet.*c2+c1;
         Cu=1./CSFet;
         Cu=Cu*ones(1,length(C));
      end
      if nolin==1
         lonf=length(fe);
         fe=fe*ones(1,length(C));
         k=(-0.079389*log10(fe)+0.322725);
         n=0.84*((fe.^1.7)./(0.7.^1.7+fe.^1.7));
         C=ones(lonf,1)*C;
         IAFet=1./(Cu+(((k.*(C.^n))-Cu).*(C./((Cu./k).^(1./n)+C))));
      else
         IAFet=1./Cu;
      end
   else
      fec=facfrec*fe;
      CSFet=(250/1896.4)*((ft+0.1*fec).*fec*4*pi^2).*(6.1+7.3*abs(log((ft+0.1*fec)./(3*fec))).^3).*exp(-(4*pi*((ft+0.1*fec)+2*fec)/45.9));
      CSFet=0.000001*(CSFet==0)+CSFet;
      c1=(1./CSFet)>1;
      c2=(1./CSFet)<1;
      CSFet=CSFet.*c2+c1;
      Cu=1./CSFet;
      Cu=Cu*ones(1,length(C));
   %   Cu=1./CSFet;
   %   Cu=Cu.*(Cu<1)+(Cu>=1);
   %   Cu=Cu*ones(1,length(C));
      if nolin==1
         fe=facfrec*fe;
         lonf=length(fe);
         fe=fe*ones(1,length(C));
         k=(-0.079389*log10(fe)+0.322725);
         n=0.84*((fe.^1.7)./(0.7.^1.7+fe.^1.7));
         C=ones(lonf,1)*C;
         IAFet=1./(Cu+(((k.*(C.^n))-Cu).*(C./((Cu./k).^(1./n)+C))));
      else
         IAFet=1./Cu;
      end
   end

if ((nolini(1)==0)&(nolini(2)==1))
   s=size(IAFet);
   iafet=sum(IAFet')';
   IAFet=iafet*ones(1,s(2));
end